#include "caur.h"
#include <stdio.h>
#include <stdlib.h>
#include <curl/curl.h>
#include <sys/stat.h>

size_t write_data(void *ptr, size_t size, size_t nmemb, FILE *stream) {
    size_t written = fwrite(ptr, size, nmemb, stream);
    return written;
}

int download_package(const char *package_name, const char *cache_dir) {
    CURL *curl;
    FILE *fp;
    CURLcode res;
    char url[256];
    char outfilename[512]; 

    // Construct the URL and output file path
    snprintf(url, sizeof(url), "%s%s%s", AUR_PACKAGE_URL, package_name, AUR_PACKAGE_URL_END);
    snprintf(outfilename, sizeof(outfilename), "%s/%s%s", cache_dir, package_name, AUR_PACKAGE_URL_END);

    curl = curl_easy_init();
    if (curl) {
        fp = fopen(outfilename, "wb");
        if (fp == NULL) {
            fprintf(stderr, "Failed to open file %s\n", outfilename);
            return 1;
        }

        curl_easy_setopt(curl, CURLOPT_URL, url);
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
        res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        fclose(fp);

        if (res != CURLE_OK) {
            fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
            return 1;
        }

        // Check if the file was downloaded correctly
        struct stat st;
        if (stat(outfilename, &st) != 0) {
            fprintf(stderr, "Downloaded file %s not found.\n", outfilename);
            return 1;
        }

        printf("Downloaded file %s successfully, size: %ld bytes\n", outfilename, st.st_size);
    } else {
        fprintf(stderr, "Failed to init curl\n");
        return 1;
    }

    return 0;
}
